﻿using System;
using System.Collections.ObjectModel;
using System.IO;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;

namespace ThinkGeoSaveShapeDbfColumnsProblem
{
    internal class ShapeFiles
    {
        public static void SaveShapeFile(FeatureLayer fl, string shapeName, ManagedProj4Projection proj4)
        {
            // Get the columns
            Collection<string> allColumns = new Collection<string>();

            fl.Open();
            foreach (FeatureSourceColumn column in fl.FeatureSource.GetColumns())
            {
                if (column.ColumnName == "Shape" || column.ColumnName == "SHAPE" || column.ColumnName == "OBJECTID")
                    continue;

                allColumns.Add(column.ColumnName);
                //allColumns.Add(new DbfColumn(column.ColumnName, colType, column.MaxLength, 0));
            }

            // Get they type of the shape file and output it.  I.e. it may be a Polygon or Points or Area.
            WellKnownType shapeType = fl.FeatureSource.GetFirstFeaturesWellKnownType();
            fl.Close();
            ShapeFileType shapeFileType = new ShapeFileType();

            switch (shapeType)
            {
                case WellKnownType.Polygon:
                    {
                        shapeFileType = ShapeFileType.Polygon;
                        break;
                    }
                case WellKnownType.Line:
                    {
                        shapeFileType = ShapeFileType.Polyline;
                        break;
                    }
                case WellKnownType.Multiline:
                    {
                        shapeFileType = ShapeFileType.Polyline;
                        break;
                    }
                case WellKnownType.Multipoint:
                    {
                        shapeFileType = ShapeFileType.Multipoint;
                        break;
                    }
                case WellKnownType.Point:
                    {
                        shapeFileType = ShapeFileType.Point;
                        break;
                    }
                case WellKnownType.Multipolygon:
                    {
                        shapeFileType = ShapeFileType.Polygon;
                        break;
                    }
            }

            if (Directory.Exists("../../FileGDB.gdb"))
            {
                Directory.Delete("../../FileGDB.gdb", true);
            }

            FileGeoDatabaseFeatureSource.CreateFileGeoDatabase("../../FileGDB.gdb");
            FileGeoDatabaseFeatureSource.CreateTable("../../FileGDB.gdb", "testtable", WellKnownType.Multipolygon, allColumns);

            FileGeoDatabaseFeatureLayer fileGDBLayer = new FileGeoDatabaseFeatureLayer("../../FileGDB.gdb", "testtable");

            fl.Open();
            proj4.Open();
            foreach (Feature feature in fl.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns))
            {
                fileGDBLayer.Open();
                try
                {
                    Feature convertedFeature = proj4.ConvertToInternalProjection(feature);

                    fileGDBLayer.EditTools.BeginTransaction();
                    fileGDBLayer.EditTools.Add(convertedFeature);
                    TransactionResult result = fileGDBLayer.EditTools.CommitTransaction();
                }
                finally
                {
                    fileGDBLayer.Close();
                }
            }
            fl.Close();

            MessageBox.Show("Convert completed!");
        }
    }
}